<?php
/**
 * $Id: LinksManagerSQLite.php 410 2016-10-11 13:04:28Z robert0609 $
 *
 * Description of LinksManagerSQLite
 *
 * @author robert0609
 */

require_once 'includes/classes/database_sqlite.php';

class CLinksManagerSQLite extends CDataBaseSQLite {

	//put your code here
	protected $m_bConfigDefined = false;

	function __construct($install_tables = false) {
		if((!defined('INI_DB_SQLITE_DATABASE')) ||
			(!defined('INI_GROUP_PERMISSIONS'))) {
			self::addMsg('INI defines missing.');
			return;
			} // if
		$this->m_sGroup = INI_GROUP_PERMISSIONS;
		if(!is_dir(LM_LOGS_DIR)) {
			mkdir(LM_LOGS_DIR, 0660);
			} // if
		self::$m_csLogFilename = LM_LOG_FILE;
		$this->m_bConfigDefined = false;

		parent::__construct(LM_FS_SQLITE_DIR . INI_DB_SQLITE_DATABASE,LM_LOG_FILE);

		if(!$install_tables) $install_tables = $this->m_bNew;
		$this->init($install_tables);
		} // __construct()

	function __destruct() {
		// clean before I go home
		parent::__destruct();
		} // __destruct()

	protected function init($install_tables) {
		if($install_tables === true) {
			if(!$this->installDataBase()) {
				die('ERROR: Failed to install tables.');
				} // if
			} // if

		// setup configuration table defines
		if((!$this->m_bConfigDefined) &&
			($config_result = $this->db_query("select lm_config_key, lm_config_value from lm_configs"))) {
			$this->m_bConfigDefined = true;
			while ($config = $this->db_fetch_array($config_result)) {	// read in config
				$key = $config['lm_config_key'];
				$val = $config['lm_config_value'];

				// and define
				switch(strtolower($val)) {
				case '0':	// that what false becomes
				case 'false':
					define($key,false);
					break;
				case '1':	// thats what true becomes
				case 'true':
					define($key,true);
					break;
				default:
					define($key,$val);
					break;
					} // switch
				} // while
			$this->db_free_result($config_result);
			} // if
		} // init()

	protected function updateDBversion() {
			$fields = array();
			$fields['lm_config_value'] = LM_PROJECT_VERSION;
			if(!$this->db_perform('lm_configs',$fields,'update',"lm_config_key = 'LMC_VERSION'")) {
				Clm::addMsg('DB update to ' . LM_PROJECT_VERSION . " failed");
				$this->logEvent('ERROR: DB update to ' . LM_PROJECT_VERSION . " failed.");
				return false;
				} // if
			Clm::addMsg('DB updated to ' . LM_PROJECT_VERSION, 'info');
			$this->logEvent('INFO: DB updated to ' . LM_PROJECT_VERSION);
			return true;
			} // updateDBversion()

	public function checkDBversion($rebuild = false) {
		if((!$rebuild) && (defined('LMC_VERSION')) && (LMC_VERSION == LM_PROJECT_VERSION)) return true;	// same version

		Clm::addMsg('Checking ' . LM_PROJECT_SHORTNAME . ' ' . LM_PROJECT_VERSION . ' database for updates.','info');

		Clm_install::clean_DB();

		if($this->installDatabase((!$rebuild ? 'lm_configs':''),false ,false)) { // update db configs
			if($rebuild) Clm_install::rebuild_from_settings();
			return $this->updateDBversion();
			} // if

		Clm::addMsg('DB update at ' . LM_PROJECT_VERSION . " failed (not checked).");
		$this->logEvent('ERROR: DB update at ' . LM_PROJECT_VERSION . " failed (not checked).");
		return false;
		} // checkDBversion()

	protected function update_row_data($table,$columns,$row,$updateable_columns,$key_column)	{ // from a virtual function
		if(empty($updateable_columns)) return true;	// nothing can changed
		$upd_cols = explode(',',$updateable_columns);
		if(count($upd_cols) < 1) return true;	// nothing to do

		$colnames = array_keys($columns);
		$key_idx = 0;	// assume it's the id col
		if(!empty($key_column)) {
			$key_idx = array_search($key_column, $colnames);
			if($key_idx === false) return false;
			} // if

		$idx = 0;
		$fields = array();
		if((int)$this->get_row_count_in_table($table,$key_column . ' = ' . $row[$key_idx]) > 0) {
			$action = 'update';
			} // if
		else $action = 'insert';
		foreach($columns as $col => $type) {
			if($idx >= 3) {	// let autoincrement and datatime do its thing
				if($action == 'insert') $fields[$col] = $row[$idx];
				else if(in_array($col, $upd_cols)) {
					$fields[$col] = $row[$idx]; // add values to update to $sql_query
					} // if
				} // if
			$idx++;
			} // foreach

		if($action == 'update') {
			if(!$this->db_perform($table,$fields,$action,$key_column . ' = ' . $row[$key_idx])) return false;
			} // if
		else if($action == 'insert') {
			if(!$this->db_perform($table,$fields,$action)) return false;
			} // else

		return true;
		} // update_row_data()

	public function installDatabase($table = '', $verbose = false, $drop = false) {
		$cLMI = new Clm_install();
		$install_scripts = $cLMI->get_installDBscripts();

		$res = $this->install_db_tables($table,$install_scripts,true,$drop);
		new Clm_xml_sitemap();	// do sitemap
		return $res;
		} // installDatabase()

} // CLinksManagerSQLite
