<?php

// $Id: database_common.php 180 2015-03-18 06:04:55Z robert0609 $
// Copyright BRAEWORKS (R) 2013

/**
 * Description of database
 *
 * @author robert0609
 */

class CDataBaseCommon extends CLogger {

	//put your code here
	protected $m_sDatabase = '';		// the database name used
	protected $m_bOpen = false;		// true when db link open
	private $username = '';
	private $password = '';
	protected $m_sGroup = '';	// group permission for web server created files
	public $m_bSaveDBtransactions = false;
	public $m_sVersion = '';
	
	function __construct($log = '') {
		parent::__construct($log);
		} // __construct()

	function __destruct() {
		parent::__destruct();
		} // __destruct()

	protected function onError($error) {
		$this->logEvent('DB_ERROR ' . $this->m_sDatabase . ' - ' . $error);
		self::addMsg('DB_ERROR ' . $this->m_sDatabase . ' - ' . $error);
		if(INI_DB_DIE_ON_ERRORS_BOOL != true) return false;	// for short cut back
		echo self::getMsgs();
		die();
		} // onError()

	public static function array_trimCallback(&$str,$key) { $str = trim($str); } // array_trimCallback()
	public static function array_trim(&$array) { return array_walk_recursive($array,'self::array_trimCallback'); } // array_trim()

	public function get_fs_group() { return $this->m_sGroup; } // get_fs_group()

	public function db_get_name() { return $this->m_sDatabase; } // db_get_name()

	public function db_output($string) {
		$string = html_entity_decode($string,ENT_COMPAT,INI_CHAR_SET);
		return htmlspecialchars($string,ENT_COMPAT,INI_CHAR_SET);
	} // db_output()

	protected function db_sanitize_string($string) {
		$string = ereg_replace(' +', ' ', trim($string));
		return preg_replace("/[<>]/", '_', $string);
	} // db_sanitize_string()

	public function db_prepare_input($string) {
		if (is_string($string)) {
			return trim(db_sanitize_string(stripslashes($string)));
		} // if
		elseif (is_array($string)) {
			reset($string);
			while (list($key, $value) = each($string)) {
				$string[$key] = db_prepare_input($value);
			} // while
			return $string;
		} // elseif
		else {
			return $string;
		} // else
	} // db_prepare_input()

	public function db_prepare_search_keyword($keyword) { // added by BF
		$keyword = db_prepare_input($keyword);
		if (empty($keyword))
			return $keyword;

		// ambiguous prefixes and suffixes that may hinder search
		$prefixes = array('[', ']', '{', '}', '\\', '/', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', '=', '|', '~', '?', '<', '>', '"', "'", '-', ';', ':', '.', ','); // single letter at this stage
		$suffixes = array('[', ']', '{', '}', '\\', '/', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', '=', '|', '~', '?', '<', '>', '"', "'", '-', ';', ':', '.', ',', 's', 'S'); // single letter at this stage

		if (is_array($keyword)) {
			reset($keyword);
			while (list($key, $value) = each($keyword)) {
				$keyword[$key] = db_prepare_search_keyword($value);
			}
		} // if
		else {
			foreach ($prefixes as $p) {
				if (strlen($keyword) < 1)
					break;
				if (substr($keyword, 0, 1) == $p)
					$keyword = substr($keyword, 1);
			} // foreach
			foreach ($suffixes as $s) {
				if (strlen($keyword) < 1)
					break;
				if (substr($keyword, -1, 1) == $s)
					$keyword = substr($keyword, 0, -1);
			} // foreach
		} // else
		return $keyword;
	} // db_prepare_search_keyword()

	protected function set_time_limit($limit) {
		// dummy function
	} // set_time_limit()


	function backupDatabase($backupFolder = '', $gzipFlg = false, $downloadFlg = false) {
		if(!$this->is_ok()) return false;
		// dummy function
		return true;
	} // backupDatabase()

	function restoreDatabase($fileURI, $uploadElement = null) {
		if(!$this->is_ok()) return false;
		// dummy function
		return true;
		} // restoreDatabase()

	protected function installTable($table,$params) {
		if(!$this->is_ok()) return false;
		// dummy function
		return true;
		} // installTable()

	protected function install_db_tables($only_table, $install_scripts, $verbose = false, $drop = false) {
		if(!$this->is_ok()) return false;
		// dummy function
		return true;
		} // install_db_tables()

} // CDataBaseCommon

?>
