<?php

// $Id: database_sqlite.php 394 2016-04-28 10:34:06Z robert0609 $
// Copyright BRAEWORKS (R) 2013

/**
 * Description of database
 *
 * @author robert0609
 */

// echo sqlite_libversion();

require_once 'includes/classes/database_common.php';

class CDataBaseSQLite extends CDataBaseCommon {

	//put your code here
	protected $m_sDatabase = '';	// the database file name used
	private $m_hSQL = null;			// the sqlite database resource handle
	protected $m_bOpen = false;		// true when db m_hSQL open
	public $m_bSaveDBtransactions = false;
	public $m_bNew = false;	// true if db created

	function __construct($database,$log = '') {
		parent::__construct($log);
		// set up
		$this->db_connect($database);
	} // __construct()

	function __destruct() {
		// clean before I go home
		$this->db_close();
		parent::__destruct();
	} // __destruct()

	public function is_ok() {
		if(!$this->m_hSQL) return false;
		if(!$this->m_bOpen) return false;

		return true;
		} // is_ok()

	public function is_table_present($table) {
		if(!$this->is_ok()) return false;
		if($mr_result = $this->db_query('SELECT * FROM sqlite_master;')) {
			while($mr = $this->db_fetch_array($mr_result)) {
				if(($mr['type'] == 'table') &&
					($mr['tbl_name'] == $table)) return true;
				} // while
			} // if
		return false;
		} // is_table_present()

	public function get_columns_list($table, $default_values = false) {	// return column names array
		if(!$this->is_ok()) return false;
		if($mr_result = $this->db_query('SELECT * FROM sqlite_master;')) {
			while($mr = $this->db_fetch_array($mr_result)) {
				if(($mr['type'] == 'table') &&
					($mr['tbl_name'] == $table)) {
					$list = array();
					$a = preg_split('/' . $table . ' ?\(|' . $table. '" ?\(/', $mr['sql']);
					$b = preg_split('/\)$/', $a[1]);
					$cols = explode(',',$b[0]);
					foreach($cols as $col) {
						$col = trim(strtolower($col));
						$c = explode(' ',$col);
						self::array_trim($c);
						$k = trim($c[0]);
						if(!$default_values) $list[] = trim($c[0]);
						else {	// extract default values
							$j = 1;
							$t = strtolower(trim($c[$j]));
							$j++;
							if((isset($c[$j])) && ($c[$j] == 'integer')) {
								$t .= ' ' . $c[$j];
								$j++;
								} // if
							$v = '';
							if((isset($c[$j])) &&
								(trim($c[$j]) == 'default') &&
								(isset($c[($j + 1)]))) {
								$j++;
								$d = '';
								for($i = $j, $n = count($c); $i < $n; $i++) {

									if(!empty($d)) $d .= ' ';
									$d .= trim($c[$i]);
									} // for
								if(!preg_match('/not null/i', $d)) {
									switch($t) {
									case 'int':
									case 'bigint':
									case 'short integer':
									case 'unsigned integer':
									case 'short':
									case 'unsigned':
									case 'integer':
									case 'boolean':
										$v = (int)$d;
										break;
									case 'float':
										$v = (float)$d;
										break;
									case 'double':
										$v = (double)$d;
										break;
									default:
										if(preg_match('/varchar|character|text/', $t)) {
											$v = preg_replace('/^[\'\"](.*)[\'\"]$/', '$1', $d);
											} // if
										else if(preg_match('/numeric/', $t)) {
											$v = (double)$d;
											} // if
										else $v = '';
										break;
										} // switch
									} // if
								} // if
							$list[trim($c[0])] = $v;
							} // else
						} // foreach
					return $list;
					} // if
				} // while
			} // if
		return false;
		} // get_columns_list()

	public function get_columns_type($table) {	// returns column names with type array
		if(!$this->is_ok()) return false;
		$types = sqlite_fetch_column_types($table,$this->m_hSQL);	// @TODO implement

		if($mr_result = $this->db_query('SELECT * FROM sqlite_master;')) {
			while($mr = $this->db_fetch_array($mr_result)) {
				if(($mr['type'] == 'table') &&
					($mr['tbl_name'] == $table)) {;
					$types = array();
					$a = preg_split('/' . $table . ' ?\(/', $mr['sql']);
					$b = preg_split('/\)$/', $a[1]);
					$cols = explode(',',$b[0]);
					foreach($cols as $col) {
						$c = explode(' ',trim($col));
						$k = $c[0];
						$v = '';
						for($i = 1, $n = count($c); $i < $n; $i++) $v .= $c[$i] . ' ';
						$types[$k] = trim($v);
						} // foreach
					return $types;
					} // if
				} // while
			} // if
		return false;
		} // get_columns_type()

	public function get_row_count_in_table($table,$where = '') {
		if(!$this->is_ok()) return false;
		if(!$this->is_table_present($table)) return false;	// no table
		$test_sql = "select count(*) as total from " . $table .
			(!empty($where) ? ' where ' . $where:'');
		if($test_result = $this->db_query($test_sql)) {
			if($test = $this->db_fetch_array($test_result)) {
				return $this->db_output($test['total']);
				} // if
			} // if
		return false; // no data, error
		} // get_row_count_in_table()

	public function is_data_in_table($table,$colname,$data) {
		if(!$this->is_ok()) return false;
		if(!$this->is_table_present($table)) return false;	// no table
		$test_sql = "select ". $colname . " from " . $table .
			" where " . $colname . " =  " . (is_numeric($data) ? $data:"'" . $this->db_input($data) . "'");
		if($test_result = $this->db_query($test_sql)) {
			if($test = $this->db_fetch_array($test_result)) {
				return true;	//$test[$colname];	// its there
				} // if
			} // if
		return false; // no data match
		} // is_data_in_table()

	public function get_data_in_table($table,$colname,$where) {
		if(!$this->is_ok()) return false;
		if(!$this->is_table_present($table)) return false;	// no table
		$test_sql = "select ". $colname . " from " . $table .
			" where " . $where;
		if($test_result = $this->db_query($test_sql)) {
			if($test = $this->db_fetch_array($test_result)) {
				return $test[$colname];
				} // if
			} // if
		return false; // no data match
		} // get_data_in_table()

	protected function checkAddDBtable($table) {
		if(!$this->is_ok()) return false;
		if($this->is_table_present($table)) return true;
		if ($this->db_query('CREATE TABLE ' . $table . ';')) return true;
		return false;
		} // checkAddDBtable()

	protected function checkAddDBcolumn($table, $column, $type) {
		if(!$this->is_ok()) return false;
		$bFound = false;
		if (($test_result = $this->db_query("PRAGMA table_info('" . $table . "');"))) {
			while ($test = $this->db_fetch_array($test_result)) {
				if ($test['name'] == $column) {
					$bFound = true;
					break;
					} // while
			} // while
			if ($bFound == false) {
				if(!$this->db_query('ALTER TABLE ' . $table . ' ADD COLUMN ' . $column . ' ' . $type . ';')) { // add
					return false;
					} // if
				} // if
			return true;
			} // if
		return false;
		} // checkAddDBcolumn()

	public function db_connect($database) {
		if($this->m_bOpen)	$this->db_close ();
		if(!function_exists('sqlite_libversion')) {
			if(class_exists('SQLite3')) {
				require_once('includes/sqlite3.php');
				} // if
			} // if
		$this->m_sVersion = sqlite_libversion();

		if(!is_readable($database)) {
			self::addMsg('Creating new database "' . basename($database) . '".','warning');
			$this->m_bNew = true;
			} // if
		else if(!is_writable($database)) {
			return $this->onError('Unable to write to "' . basename($database) . '".');
			} // if

		// copy details for error checks
		$this->m_sDatabase = $database;

		if(!function_exists("sqlite_libversion")) {
			$this->onError("ERROR: PHP Sqlite not installed or not functioning");
			die("ERROR: PHP Sqlite not installed or not functioning");
			exit(100);
			} // if

		$error_msg = '';
		$this->m_hSQL = sqlite_open($this->m_sDatabase,0664,$error_msg);
		if (!$this->m_hSQL) {
			return $this->onError('Could not open "' . $this->m_sDatabase . '": ' . $error_msg);
			} // if
		if($this->m_bNew) {
			@chmod($this->m_sDatabase,0664);
			if(!empty($this->m_sGroup)) chgrp($this->m_sDatabase,$this->m_sGroup);
			} // if
		$this->m_bOpen = true;
	} // db_connect()

	public function db_close() {
		if ($this->m_hSQL) {
			sqlite_close($this->m_hSQL);
			$this->m_hSQL = null;
		} // if
		$this->m_bOpen = false;
	} // db_close()

	public function db_error($query) {
		if(!$this->is_ok()) return false;
		$errno = sqlite_last_error($this->m_hSQL);
		$error = sqlite_error_string($this->m_hSQL);	//($errno);
		$this->logEvent('DB_ERROR ' . $this->m_sDatabase . ' - ' . $errno . ' - ' . $error . ' - ' . $query);
		return $this->onError($this->m_sDatabase . ' - ' . $errno . ' - ' . $error . LM_NL . $query . LM_NL . '[DB STOP]');
		} // db_error()

	public function db_perform($table, $data, $action = 'insert', $where_params = '',$unchecked_cols = null) {
		if(!$this->is_ok()) return false;
		reset($data);
		if ($action == 'insert') {
			$query = 'insert into ' . $table . ' (';
			while (list($columns, ) = each($data)) {
				$query .= $columns . ', ';
			} // while
			$query = substr($query, 0, -2) . ') values (';
			reset($data);
			while (list(, $value) = each($data)) {
				$value = preg_replace("/^'|'$/",'',$value);
				switch ((string) $value) {
					case 'now()':
						$query .= "DATETIME('NOW'), ";
						break;
					case 'null':
						$query .= 'null, ';
						break;
					default:
						if(is_bool($value)) $query .= ($value ? 1:0) . ', ';
						else if(is_numeric($value)) $query .= $value . ', ';
						else $query .= '\'' . $this->db_input($value) . '\', ';
						break;
				} // switch
			} // while
			$query = substr($query, 0, -2) . ')';
		} // if
		elseif ($action == 'update') {
			if(!empty($unchecked_cols) && is_array($unchecked_cols)) {

				} // if
			$query = 'update ' . $table . ' set ';
			while (list($columns, $value) = each($data)) {
				$value = preg_replace("/^'|'$/",'',$value);
				switch ((string) $value) {
					case 'now()':
						$query .= $columns . "DATETIME('NOW'), ";
						break;
					case 'null':
						$query .= $columns . ' = null, ';
						break;
					default:
						if(is_bool($value)) $query .= $columns . ' = ' . ($value ? 1:0) . ', ';
						else if(is_numeric($value)) $query .= $columns . ' = ' . $value . ', ';
						else $query .= $columns . ' = \'' . $this->db_input($value) . '\', ';
						break;
				} // switch
			} // while
			$query = substr($query, 0, -2) . ' where ' . $where_params;
		} // elseif
		else return false;

		return $this->db_query($query);
	} // db_perform()

	public function db_query($query) {
		if(!$this->is_ok()) return false;
		if ($this->m_bSaveDBtransactions === true) {
			$this->logEvent('DB_QUERY ' . $query . "\n");
			} // if

		$result = sqlite_query($this->m_hSQL, $query);
		if ($result === false) return $this->db_error($query);

		return $result;
	} // db_query()

	public function db_fetch_array($query_result) {
		if(!$this->is_ok()) return false;
		if(!$query_result) return false;
		return sqlite_fetch_array($query_result);
	} // db_fetch_array()

	public function db_num_rows($query_result) {
		if(!$this->is_ok()) return false;
		if(!$query_result) return false;
		return sqlite_num_rows($query_result);
	} // db_num_rows()

	public function db_query_unbuffered($query) {
		if(!$this->is_ok()) return false;
		if ($this->m_bSaveDBtransactions === true) {
			$this->logEvent('DB_QUERY ' . $query . "\n");
			} // if

		$result = sqlite_unbuffered_query($this->m_hSQL, $query);
		if ($result === false) return $this->db_error($query);

		return $result;
	} // db_query_unbuffered()

	public function db_fetch_array_unbuffered($query_result,$next = true) {
		if(!$this->is_ok()) return false;
		if(!$query_result) return false;
		$result = sqlite_current($query_result);
		if(($next) && ($result)) sqlite_next($query_result);
		return $result;
	} // db_fetch_array_unbuffered()

	public function db_data_seek($query_result, $row_number) {
		if(!$this->is_ok()) return false;
		if(!$query_result) return false;
		return sqlite_seek($query_result, $row_number);
	} // db_data_seek()

	public function db_insert_id() {
		if(!$this->is_ok()) return false;
		return sqlite_last_insert_rowid($this->m_hSQL);
	} // db_insert_id()

	public function db_free_result($query_result) {
		if(!$this->is_ok()) return false;
		if(!$query_result) return false;
		return true;	// not available	SQLite_free_result($query_result);
	} // db_free_result()

//	public function db_fetch_fields($query_result) {
//		if(!$this->is_ok()) return false;
//		return SQLite_fetch_field($query_result);
//	} // db_fetch_fields()

	public function db_input($string) {
		if(!$this->is_ok()) return false;
//		if(!function_exists('sqlite_escape_string')) return $string;
//		if(is_array($string)) {
//			$str_a = array();
//			foreach($string as $k => $v) {
//				$str_a[$k] = $this->db_input($v);	// recurse thru array
//				} // foreach
//			return $str_a;
//			} // if
//		$string = sqlite_escape_string($string);
		// @TODO sort this
		$string = str_replace("\"", '', $string);
		$string = str_replace("\\", '', $string);
		$string = str_replace("'", '', $string);
		return $string;
	} // db_input()

	public function db_output($string) {	// opposite of db_input
		if(!$this->is_ok()) return false;
//		if(!function_exists('sqlite_escape_string')) return $string;
//		if(is_array($string)) {
//			$str_a = array();
//			foreach($string as $k => $v) {
//				$str_a[$k] = $this->db_output($v);	// recurse thru array
//				} // foreach
//			return $str_a;
//			} // if
//		$string = str_replace("''","'",$string);
//		$string = str_replace('\"','"',$string);
		return $string;
	} // db_output()

	protected function update_row_data($table,$columns,$row,$updateable_columns,$key_column)	{ // a virtual function
		return false;
		} // update_row_data()

	protected function installTable($table,$params, $drop = false) {
		if(!$this->is_ok()) return false;
		if((empty($table)) || (!is_array($params))) return false;

		// $params
		// 'drop' => 'false' or 'true'
		// 'columns' => array( 'col_name' => 'type/def' )
		// 'data' => array() of column data in columns order

		$ok = true;
		$table_present = $this->is_table_present($table);
		if(($table_present) &&
			(isset($params['drop'])) &&
			(($params['drop'] == 'true') || ($params['drop'] === true) || ($drop))) {
			$this->db_query("DROP TABLE " . $table . ";");
			$table_present = false;
			} // if

		// $this->db_query("SET @saved_cs_client     = @@character_set_client;");	// save language
		// $this->db_query("SET character_set_client = utf8;");	// set language

		if((is_array($params['columns'])) && (count($params['columns'] > 0))) {
			$columns = $params['columns'];

			if(!$table_present) {
				$sql_query = 'CREATE TABLE ' . $table;
				$sql_query .= ' (';
				$cnt = 0;
				foreach ($columns as $name => $type) {
					if($cnt > 0) $sql_query .= ', ';
					$sql_query .= "" . $name . " " . $type;
					$cnt++;
					} // foreach
				$sql_query .= ');';
				if(!$this->db_query($sql_query)) {
					$ok = false;
					} // if
				} // if
			else {	// check all columns are there
				foreach ($columns as $name => $type) {
					if(!$this->checkAddDBcolumn($table, $name, $type)) $ok = false;
					} // foreach
				} // else

			if(($ok) && (isset($params['functions']))) {	// create triggers, filters and functions
				foreach($params['functions'] as $k => $v) {
					// check if function ($k) is already there first
					if($test_query = $this->db_query("SELECT * FROM sqlite_master WHERE tbl_name = '" . $table . "' AND TYPE = 'trigger';")) {
						$found = false;
						while($test = $this->db_fetch_array($test_query)) {
							if($test['name'] == $k) {
								$found = true;
								break;
								} // if
							} // while
						if($found) continue;
						} // if
					$sql_query = $v;
					if(!$this->db_query($sql_query)) {
						$ok = false;
						break;
						} // if
					} // foreach
				} // if

			if(($ok) && (isset($params['data'])) && (is_array($params['data'])) && (count($params['data'] > 0))) {
				$data = $params['data'];	// in the same order as the columns
				if(!$table_present) {	// empty table
					foreach($data as $row) {
						$sql_query = "INSERT INTO " . $table . " VALUES (" . implode(',',$row) . ");";
						if(!$this->db_query($sql_query)) $ok = false;
						} // foreach
					} // if
				else {	// add / update columns
					$columns = $params['columns']; // assumes columns[0] is the id name and row[0] is the value
					$colnames = array_keys($columns);
					foreach($data as $row) {
						if((isset($params['updateable_columns'])) && (!empty($params['updateable_columns'])) &&
							((isset($params['key_column'])) && (!empty($params['key_column'])))) {
							if(!$this->update_row_data($table,$columns,$row,$params['updateable_columns'],$params['key_column'])) $ok = false;
							} // if
						else if(((!isset($params['key_column'])) || (empty($params['key_column']))) &&
							((int)$row[0] > 0) &&	// ??? what for
							(!$test_query = $this->db_query('SELECT * FROM ' . $table . ' WHERE ' . $colnames[0] . ' = ' . $row[0] . '')) ||
							($this->db_num_rows($test_query) < 1)) {
							$sql_query = 'INSERT INTO ' . $table . ' VALUES (' . implode(',',$row) . ');';
							if(!$this->db_query($sql_query)) $ok = false;
							} // if
						} // foreach
					} // else
				} // if
			} // if
		else $ok = false;

		return $ok;
		} // installTable()

	protected function install_db_tables($only_table, $install_scripts, $verbose = false, $drop = false) {
		if(!$this->is_ok()) return false;
		$ok = true;
		foreach($install_scripts as $table => $params) {
			if((strlen($only_table) > 0) &&
				($table != $only_table)) continue;

			if($verbose) self::addMsg(($drop ? 'Installing table ':'Checking table ') . $table,'info');
			if(!$this->installTable($table,$params,$drop)) {
				if($verbose) self::addMsg('Failed to ' . ($drop ? "install":"check") . $table . ' in ' . $this->db_get_name(), 'error');
				$this->logEvent('Failed to ' . ($drop ? "install":"check") . $table . ' in ' . $this->db_get_name());
				$ok = false;
				} // if
			} // foreach
		if($verbose) {
			if($ok) self::addMsg(($drop ? "Installed":"Checked") . " " . basename($this->db_get_name()) . " tables - success", 'success');
			else self::addMsg("Failed to " . ($drop ? "install":"check") . " " . basename($this->db_get_name()) . " tables - failed");
			} // if
		return $ok;
		} // install_db_tables()

	function backupDatabase($backupFolder = '', $gzipFlg = false, $downloadFlg = false) {
		if(!$this->is_ok()) return false;
		// @TODO
		// like sqlite database ".dump" > filename
		return true;
	} // backupDatabase()

	function restoreDatabase($fileURI, $uploadElement = null) {
		if(!$this->is_ok()) return false;
		// @TODO
		return true;
		} // restoreDatabase()

} // CDataBaseSQLite

?>
