<?php

/**
 * $Id: lm.php 419 2016-10-18 12:30:56Z robert0609 $
 *
 * Description of lm
 * The top class for links_manager
 *
 * @author robert0609
 */

class Clm extends Clm_html {

	//put your code here
	private static $page_start_time = 0;
	private static $page_end_time = 0;
	private static $group_manager = false;	// time saver

	public static $version_str = false;

	function __construct() {

		self::$page_start_time = microtime(true);

		// complete the defines from the ini file
		$this->define_install_settings();
		$this->php_mods = get_loaded_extensions();
		if((INI_ALLOW_HTTP_COMPRESSION_BOOL) &&
			(!in_array('eAccelerator', $this->php_mods))) {
			ini_set("zlib.output_compression", 'On');
			} //if
		else ini_set("zlib.output_compression", 'Off');
		if(!defined('INI_TIMEZONE_TZ')) date_default_timezone_set("Australia/Melbourne");	// set a default
		else date_default_timezone_set(INI_TIMEZONE_TZ);
		if((!self::do_lm_warnings()) &&
			(self::getMsgsCount() > 0) &&
			(INI_DB_DIE_ON_ERRORS_BOOL)) {
			echo self::getMsgs();
			echo '<br>';
			die("Failed installation checks.");
			} // if

		// debug definitions
		if((defined('CLI_MODE')) && (CLI_MODE)) {
			ini_set('output_buffering', 'Off');
			ini_set('display_errors','On');
			ini_set('display_startup_errors','On');
			ini_set('html_errors', 'Off');
			} // if
		else if(INI_DEBUG_BOOL) {
			ini_set('output_buffering', 'Off');
			ini_set('display_errors','On');
			ini_set('display_startup_errors','On');
			ini_set('html_errors', 'On');

			} // if
		else {
			ini_set('output_buffering', 'On');
			ini_set('display_errors','Off');
			ini_set('display_startup_errors','Off');
			ini_set('html_errors', 'Off');
			} // else

		self::$cDB = new CLinksManagerSQLite();	// also reads in configuration definitions
		self::$cDB->checkDBversion(self::$cDB->m_bNew);	// if new DB will rebuild install and CSS files

		if((!defined('CLI_MODE')) || (!CLI_MODE)) {
			session_name(LM_SESSION_NAME);
			session_save_path(LM_FS_BASE_DIR . 'sessions');

			if((INI_ALLOW_COOKIE_LOGIN_BOOL) &&
				(isset($_COOKIE[LM_LOGIN_COOKIE_NAME])) &&
				(!empty($_COOKIE[LM_LOGIN_COOKIE_NAME]))) {
				// session lifetime for remembered logins
				session_set_cookie_params(LM_USER_COOKIE_TIMEOUT_SECS);
				} // if
			else session_set_cookie_params(LM_USER_TIMEOUT_SECS);

			session_start();

			if((isset($_SESSION['user']['timeout'])) &&
				($_SESSION['user']['timeout'] < time())) {
				// login timeout
				unset($_SESSION['user']);
				unset($_SESSION['action']);
				unset($_GET['action']);
				unset($_POST['action']);
				} // if

			if((INI_ALLOW_COOKIE_LOGIN_BOOL) &&
				(isset($_COOKIE[LM_LOGIN_COOKIE_NAME])) &&
				(!empty($_COOKIE[LM_LOGIN_COOKIE_NAME])) &&
				(!isset($_SESSION['user']))) {
				// do cookie login
				$ids = explode(':',$_COOKIE[LM_LOGIN_COOKIE_NAME]);
				if((self::get_user_cookie_id($ids[0]) == $_COOKIE[LM_LOGIN_COOKIE_NAME]) &&
					(is_readable(session_save_path() . '/sess_' . $ids[1]))) {
					self::log_in_user_id($ids[0],true);
					$url = LM_SSL_URL . 'index.php';
					header('Location: ' . $url);
					exit (0);
					} // if
				} // if

			if((INI_ALWAYS_LOGIN_BOOL) &&
				(basename($_SERVER['PHP_SELF']) != 'login.php') &&
				((!isset($_GET['action'])) || ($_GET['action'] != 'login'))) {
				if(!self::is_user_logged_in()) {
					$url = LM_SSL_URL . 'login.php';
					header('Location: ' . $url);
					exit (0);
					} // if
				} // if
			else {
				if(self::is_user_logged_in()) {
					// still in time
					$_SESSION['user']['timeout'] = time() + LM_USER_TIMEOUT_SECS;	// reset timeout in seconds
					if((INI_ALLOW_COOKIE_LOGIN_BOOL) &&
						(isset($_COOKIE[LM_LOGIN_COOKIE_NAME])) &&
						(!empty($_COOKIE[LM_LOGIN_COOKIE_NAME]))) {
						setcookie(LM_LOGIN_COOKIE_NAME, $_COOKIE[LM_LOGIN_COOKIE_NAME], time() + LM_USER_COOKIE_TIMEOUT_SECS,LM_WS_DIR);	// 90 days
						} // if
					} // if
				} // else

			// logged in ?
			self::$block_html = INI_PAGE_STYLE_BLOCK_BOOL;
			if((!INI_DEBUG_BOOL) && (self::is_user_logged_in())) {
				self::$block_html = ($_SESSION['user']['inline_html'] ? false:true);
				} // if
			} // if

		$this->do_globals();
		parent::__construct();

		// look for safari ?
		// eg Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25
		$users_agent_str = $this->get_user_agent();
		self::$tablet_flg = $this->is_mobile_device($users_agent_str);
		self::$tiny_flg = $this->is_tiny_device($users_agent_str);

		if((self::$tablet_flg) && (self::$ssl_required)) self::$block_html = false;
		if(self::$tiny_flg) self::$block_html = false;

		// style override ?
		if(preg_match('/^Mozilla\/4\../i', $users_agent_str)) {
			self::$block_html = false;	// IE 6 and others
			self::$moz_4_flg = true;	// compatibility issues
			self::$browser_html5 = false;
			} // if
		else if(preg_match('/Konqueror\/4\.3/i', $users_agent_str)) {
			// Konqueror 4.3.4 is full of problems, bugs and compatibility issues
			self::$moz_4_flg = true;	// try to keep it sane
			self::$browser_html5 = false;	// NOT
		} else self::$moz_4_flg = false;

		if(preg_match('/chrome\//i', $users_agent_str)) self::$chrome_flg = true;
		else self::$chrome_flg = false;

		self::$tooltips_flg = $this->show_title_tooltips();
		self::$admin_tools_in_header = $this->is_admin_tools_in_header();
		self::$left_column = $this->show_left_column();

		self::$version_str = strip_tags(LM_PROJECT_SHORTNAME . ' ' . LM_PROJECT_VERSION);
		if(self::is_group_manager()) {
			self::$version_str .= ', Sqlite V' . self::$cDB->m_sVersion;
			if((is_dir(LM_FS_BASE_DIR . '/.svn')) &&
				(function_exists('svn_status'))) {	//	// get svn stats
				$svn = svn_status(LM_FS_BASE_DIR,(0 | SVN_ALL));
				$rev_hi = 0; $rev_lo = -1;
				foreach($svn as $s) {
					if(!isset($s['revision'])) continue;
					if($rev_hi < (int)$s['revision']) $rev_hi = (int)$s['revision'];
					if(($rev_lo < 0) || ($rev_lo > (int)$s['revision'])) $rev_lo = (int)$s['revision'];
					} // foreach
				self::$version_str .= ', SVN Revs: ' . $rev_lo . ' to ' . $rev_hi;
				} // if
			} // if
		self::$version_str .= (INI_DEBUG_BOOL ? ', (Debug)':'');
		} // __construct()

	function __destruct() {
		parent::__destruct();
		if(is_object(self::$cDB)) self::$cDB->db_close();
		} // __destruct()

// dynamic functions

	private function do_globals() {
		global $action;
			$action = '';
			if((isset($_GET['action'])) && (!empty ($_GET['action']))) {
				$action = $_GET['action'];
				$_SESSION['action'] = $action;
				} // if
			else if((isset($_POST['action'])) && (!empty ($_POST['action']))) {
				$action = $_POST['action'];
				$_SESSION['action'] = $action;
				} // if
			else if(isset($_SESSION['action'])) $action = $_SESSION['action'];
		} // do_globals()

	private function is_admin_tools() {
		if((defined('CLI_MODE')) && (CLI_MODE)) return false;
		if((isset($_GET['action'])) && ($_GET['action'] == 'login')) return false;
		else if((isset($_POST['action'])) && ($_POST['action'] == 'login')) return false;
		else if(self::is_admin_user()) return true;
		else if(self::get_tool_cnt() > 0) return true;
		return false;
		} // is_admin_tools()


	protected function is_admin_tools_in_header() {
		if(!$this->is_admin_tools()) return false;
		if(self::$tiny_flg) return true;
		if(INI_LEFT_COLUMN_BOOL) return false;
		// if(self::$tablet_flg) return false;
		if(self::$moz_4_flg) return false;
		if(!self::$browser_html5) return false;
		// if(($action != 'link') && ($action != 'tool')) return false;
		return true;
		} // is_admin_tools_in_header()

	protected function show_left_column() {
		if(self::$admin_tools_in_header) return false;
		// if($this->is_admin_tools_in_header()) return false;
		if(!$this->is_admin_tools()) return false;
		if(self::$tiny_flg) return false;
		if(INI_LEFT_COLUMN_BOOL) return true;
		//if(self::$tablet_flg) return true;
		if(self::$moz_4_flg) return true;
		return false;
		} // show_left_column()

// static functions

	private static function get_user_cookie_id($user_id) {
		$sql_query = "SELECT  lm_user_id,lm_user_name,lm_user_added" .
			" FROM  lm_users" .
			" WHERE lm_user_id = " . (int)$user_id . "";
		if(($user = self::$cDB->db_query($sql_query)) &&
			($user = self::$cDB->db_fetch_array($user))) {
			$cookie_id = $user['lm_user_id'] . ':' . session_id() . ':' . sha1($user['lm_user_name'] . $user['lm_user_added']) ;
			return $cookie_id;
			} // if
		return false;
		} // get_user_cookie_id()

	protected static function inc_login_count($lm_user_id) {
		$sql_query = "SELECT  lm_user_login_count FROM  lm_users WHERE  lm_user_id = '" . (int)$lm_user_id . "'";
		if((!$result = Clm::$cDB->db_query($sql_query)) ||
			(!$user = Clm::$cDB->db_fetch_array($result))) return false;
		Clm::$cDB->db_free_result($result);
		if(!Clm::$cDB->db_query("UPDATE lm_users SET lm_user_login_count = " . ((int)$user['lm_user_login_count'] + 1) . " WHERE lm_user_id = '" . (int)$lm_user_id . "'")) return false;
		return true;
		} // inc_login_count()

	public static function log_in_user_id($user_id,$remember = false) {
		if((self::$cDB->is_table_present('lm_users')) &&
			($user_rows = self::$cDB->db_query("SELECT COUNT(*) AS total FROM  lm_users WHERE  lm_user_enabled = 1")) &&
			($user_rows = self::$cDB->db_fetch_array($user_rows)) &&
			($user_rows['total'] > 0)) {

			$sql_query = "SELECT  lm_user_id,lm_user_name,lm_user_password_md5,lm_user_admin,lm_user_group_ids,lm_user_inline_html" .
				" FROM  lm_users" .
				" WHERE  lm_user_enabled = 1 AND  lm_user_id = " . (int)$user_id . "";
			if(($user = self::$cDB->db_query($sql_query)) &&
				($user = self::$cDB->db_fetch_array($user))) {
				$user_id = $user['lm_user_id'];
				$user_name = $user['lm_user_name'];
				$user_password_md5 = $user['lm_user_password_md5'];
				$user_admin = $user['lm_user_admin'];
				$user_group_ids = $user['lm_user_group_ids'];
				$user_inline_html = $user['lm_user_inline_html'];
				$user_group_manager_ids = '';
				if(!$user_admin) { // check if in an admin group
					$group_ids = explode(':',$user_group_ids);
					$group_manager_ids = array();
					foreach($group_ids as $gid) {
						// check if an admin
						$admin = self::$cDB->get_data_in_table('lm_groups','lm_group_admin','lm_group_id = ' . $gid);
						if($admin) {
							$user_admin = true;
							break;
							} // if
						// check if a group manager
						$manager_ids = self::$cDB->get_data_in_table('lm_groups','lm_group_manager_ids','lm_group_id = ' . $gid);
						$mids = explode(':',$manager_ids);
						if(in_array($user_id, $mids)) $group_manager_ids[] = $gid;
						} // foreach
					if((!$user_admin) && (count($group_manager_ids) > 0)) $user_group_manager_ids = implode(':', $group_manager_ids);
					} // if
				$_SESSION['user'] = array(
					'id' => $user_id,
					'name' => $user_name,
					'timeout' => time() + LM_USER_TIMEOUT_SECS,	// timeout in seconds
					'admin' => $user_admin,	// administrator flag
					'group_manager_ids' => $user_group_manager_ids,
					'group_ids' => $user_group_ids,	// group ids array
					'inline_html' => $user_inline_html,
					);
				if((INI_ALLOW_COOKIE_LOGIN_BOOL) &&
					($remember)) {
					$cookie_id = self::get_user_cookie_id($user_id);
					setcookie(LM_LOGIN_COOKIE_NAME, $cookie_id, time() + LM_USER_COOKIE_TIMEOUT_SECS,LM_WS_DIR);	// 90 days
					} // if
				else if(isset($_COOKIE[LM_LOGIN_COOKIE_NAME])) {
					setcookie(LM_LOGIN_COOKIE_NAME, 0, time() - 3600,LM_WS_DIR);	// expire cookie
					} // else if
				self::inc_login_count($user_id);
				if(isset($_SESSION['BLIND_LOGIN_FAILED'])) unset($_SESSION['BLIND_LOGIN_FAILED']);
				// NOTE: redirect does not save the session
				return true;
				} // if
			} // if
		return false;
		} // log_in_user_id()

	public static function is_user_logged_in($update_time = false) {
		if(isset($_SESSION['user'])) return true;
		return false;
		} // is_user_logged_in()

	public static function is_admin_user() {
		if(self::is_user_logged_in()) {
			if((int)$_SESSION['user']['admin'] > 0) return true;
			} // if
		return false;
		} // is_admin_user()

	public static function is_group_manager($gids = -1) {
		if(self::$group_manager !== false) return (self::$group_manager ? true:false);
		self::$group_manager = 0;
		if(self::is_admin_user()) self::$group_manager = 1;
		else if((isset($_SESSION['user']['group_manager_ids'])) &&
			(!empty($_SESSION['user']['group_manager_ids'])) &&
			(self::is_user_logged_in())) {
			if(($gids == -1) && (!empty($_SESSION['user']['group_manager_ids']))) self::$group_manager = 1; // is a group manager of something
			else {
				$mids = explode(':',$_SESSION['user']['group_manager_ids']);
				$gids = explode(':',$gids);
				foreach($gids as $gid) {
					if((int)$gid == 0) { self::$group_manager = 1; break; } // all groups
					else if(in_array($gid,$mids)) { self::$group_manager = 1; break; }
					} // foreach
				} // else
			} // if
		return (self::$group_manager ? true:false);
		} // is_group_manager()

	public static function is_config_user() {
		if(self::is_admin_user()) return true;
		return self::is_group_manager(-1);
		} // is_config_user()

	public static function get_tools() {
		if(self::$tools === false) {	// not set yet
			self::$tools = array();

			$sql_query = "SELECT  lm_tool_id,lm_tool_name,lm_tool_url,lm_tool_title" .
				", lm_tool_new_page, lm_tool_enabled, lm_tool_ssl, lm_tool_group_ids, lm_tool_add_name2url" .
				" FROM  lm_tools" .
				" WHERE  lm_tool_enabled > 0" .
				" ORDER BY  lm_tool_order,lm_tool_name,lm_tool_id";
			if(($result = self::$cDB->db_query($sql_query)) &&
				(self::$cDB->db_num_rows($result) > 0)) {
				while($tool = self::$cDB->db_fetch_array($result)) {
					if(!self::check_user_group_ids($tool['lm_tool_group_ids'])) continue;
					$url = '';
					if($tool['lm_tool_ssl']) $url .= LM_SSL_URL;
					if(!$tool['lm_tool_new_page']) {
						$url .= 'index.php?action=tool&tool_id=' . (int)$tool['lm_tool_id'] . '&name=' . urlencode($tool['lm_tool_name']);
						} // if
					else {
						$url .= LM_LOCAL_TOOLS_FOLDER . $tool['lm_tool_url'] . ((int)$tool['lm_tool_add_name2url'] ? '?name=' . urlencode($tool['lm_tool_name']):'');
						} // else

					self::$tools[] = array(
						'name' => $tool['lm_tool_name'],
						'url' => $url,
						'new' => (($tool['lm_tool_new_page']) ? ' target="_blank"':''),
						'title' => (!empty($tool['lm_tool_title']) ? strip_tags($tool['lm_tool_title']):LM_LOCAL_TOOLS_FOLDER . $tool['lm_tool_url']). (($tool['lm_tool_new_page'] && LMC_SHOWINNEWTAB) ? "\n" . '(in a new tab)':''),
						);
					} // while
				} //if
			self::$cDB->db_free_result($result);
			} // if
		return self::$tools;
		} // get_tools()

	public static function get_tool_cnt() {
		if(self::$tools_cnt === false) {	// not set yet
			self::$tools_cnt = count(self::get_tools());
			} // if
		return self::$tools_cnt;
		} // get_tool_cnt()

	public static function show_stats() {
		$text = date('d M Y');
		if(LMC_SHOW_STATS) {
			$text .= ', Used: ' . number_format((memory_get_peak_usage() / 1024)) . 'kB';
			self::$page_end_time = microtime(true);	// not quite the end
			$text .= sprintf(", %0.3f seconds.", self::$page_end_time - self::$page_start_time);
			} // if
		else $text .= '&nbsp;'; // something
		return $text;
		} // show_stats()

	public static function do_google_analytics() {
		if((!INI_DEBUG_BOOL) && (strlen(INI_GOOGLE_ANALYTICS) > 4) &&
			(is_readable(LM_FS_INCLUDES_DIR . INI_GOOGLE_ANALYTICS))) {
			Clm::page_start_comment(INI_GOOGLE_ANALYTICS);
			include(LM_FS_INCLUDES_DIR . INI_GOOGLE_ANALYTICS);
			Clm::page_end_comment(INI_GOOGLE_ANALYTICS);
			} //if
		} // do_google_analytics()

} // Clm
