<?php

/**
 * $Id: lm_export.php 139 2015-03-04 12:03:22Z robert0609 $
 *
 *  * Description of lm_export
 *	* static class to provide text export in user data
 *
 * @author robert0609
 */
class Clm_export extends Clm_general {

	const delim = "\t";

	function __construct() {
		parent::__construct();
		} // __construct()

	function __destruct() {
		parent::__destruct();
		} // __destruct()

// static methods
	public static function get_tables() {
		return array(
			'lm_sections',
			'lm_links',
			);
		} // get_tables()

	public static function get_section_colnames() {
		return array(	// col names in csv
			"lm_section_name",
			"lm_section_enabled",
			"lm_section_order",
			"lm_section_columns",
			"lm_section_title",
			"lm_section_description",
			"lm_section_group_ids",
			"lm_section_image_url",
			);
		} // get_section_colnames()

	public static function get_section_index() {
		return array("lm_section_name");	// the unique index columns
		} // get_section_index()

	public static function get_link_colnames() {
		return array(	// col names in csv
			"lm_section_name",
			"lm_link_name",
			"lm_link_enabled",
			"lm_link_order",
			"lm_link_title",
			"lm_link_description",
			"lm_link_url",
			"lm_link_image_url",
			"lm_link_new_page",
			"lm_link_ssl",
			"lm_link_add_name2url"
			);
		} // get_link_colnames()

	public static function get_link_index() {
		return array("lm_section_name","lm_link_name");	// the unique index columns
		} // get_link_index()

	public static function export_sections($csv_file) {
		$colnames = self::get_section_colnames();
		if((!$fh = fopen($csv_file,'w')) ||
			(!fwrite($fh,implode(self::delim,$colnames) . "\r\n"))) {
			if($fh) fclose($fh);
			return false;
			} // if
		$sql_section_query = "SELECT lm_section_id, " . implode(',',$colnames) .
			" FROM  lm_sections ORDER BY lm_section_name;";
		if($result_section = Clm::$cDB->db_query_unbuffered($sql_section_query)) {
			while($section = Clm::$cDB->db_fetch_array_unbuffered($result_section)) {
				reset($colnames);
				$text = '';
				foreach($colnames as $col) {
					if(!empty($text)) $text .= self::delim;
					$text .= $section[$col];
					} // foreach
				fwrite($fh,$text . "\r\n");
				} // while
			} // if
		fclose($fh);
		return true;
		} // export_sections()

	protected static function import_table($table,$csv_file,$name,$colnames,$idxcols) {
		$cLMC = new Clmc();
		$file = $cLMC->get_txt_file($name, $csv_file, str_replace(LM_FS_BASE_DIR, '',$csv_file));
		if(empty($file)) {
			Cmsgs::addMsg('Failed to upload "' . $csv_file .'"');
			return false;
			} // if
		$head = ''; $fh = false;
		if((!is_readable($csv_file)) ||
			(!$fh = fopen($csv_file,'r')) ||
			(!$head = fgetcsv($fh,1000,self::delim))) {
			if($fh) fclose($fh);
			Cmsgs::addMsg('Failed to read "' . $csv_file .'"');
			return false;
			} // if
		// sort($head); sort($colnames);
		if($head != $colnames) {
			Cmsgs::addMsg('Header row in "' . $csv_file .'" does not match column in table ' . $table . '.');
			return false;
			} // if

		$head_idx = array_flip($head);
		$c_cnt = count($colnames);
		while(($data = fgetcsv($fh,10000,self::delim)) !== false) {
			reset($colnames);

			// make where strinhg
			$where = '';
			foreach($idxcols as $idx) {
				if(!empty($where)) $where .= ' AND ';
				$where .= $idx . " = '" . $data[($head_idx[$idx])] . "'";
				} // foreach

			// make values array
			$fields = array();
			for($i = 0; $i < $c_cnt; $i++) {
				$fields[($head[$i])] = $data[$i];
				} // for

			// check if new or update
			$sql_query = "SELECT * FROM " . $table . " WHERE ";
			$sql_query .= $where;


			if(($result_test = Clm::$cDB->db_query($sql_query)) &&
				(Clm::$cDB->db_num_rows($result_test) > 0)) {	// update if
				if(!Clm::$cDB->db_perform($table, $fields, 'update', $where)) {
					Cmsgs::addMsg('Failed to update row in table ' . $table . ' at ' . $where . '.');
					return false;
					} // if
				} // if
			else { // insert it
				if(!Clm::$cDB->db_perform($table, $fields, 'insert')) {
					Cmsgs::addMsg('Failed to insert row in table ' . $table . ' at ' . $where . '.');
					return false;
					} // if
				} // else
			} // while

		return true;
		} // import_table()

	public static function import_sections($csv_file,$name) {
		$colnames = self::get_section_colnames();
		$idxcols = self::get_section_index();
		return self::import_table('lm_sections', $csv_file, $name, $colnames, $idxcols);
		} // import_sections()

	public static function export_links($csv_file) {
		$colnames = self::get_link_colnames();
		if((!$fh = fopen($csv_file,'w')) ||
			(!fwrite($fh,implode(self::delim,$colnames) . "\r\n"))) {
			if($fh) fclose($fh);
			return false;
			} // if
		$sql_link_query = "SELECT l.lm_link_id," . implode(',',$colnames) .
			" FROM  lm_links as l, lm_sections as s" .
			" WHERE l.lm_link_section_id = s.lm_section_id" .
			" ORDER BY s.lm_section_name,l.lm_link_name;";
		if($result_link = Clm::$cDB->db_query_unbuffered($sql_link_query)) {
			while($link = Clm::$cDB->db_fetch_array_unbuffered($result_link)) {
				reset($colnames);
				$text = '';
				foreach($colnames as $col) {
					if(!empty($text)) $text .= self::delim;
					$text .= $link[$col];
					} // foreach
				fwrite($fh,$text . "\r\n");
				} // while
			} // if
		fclose($fh);
		return true;
		} // export_links()

	public static function import_links($csv_file,$name) {
		$colnames = self::get_link_colnames();
		$idxcols = self::get_link_index();
		return self::import_table('lm_links', $csv_file, $name, $colnames, $idxcols);
		} // import_sections()

} // Clm_export
