<?php

/**
 * $Id: lm_html.php 139 2015-03-04 12:03:22Z robert0609 $
 *
 *  * Description of lm_search
 *	* static class to provide theme, install and configuration option for display
 *	* an admin help class
 *
 * @author robert0609
 */
class Clm_search extends Clm_general {

	function __construct() {
		parent::__construct();
		} // __construct()

	function __destruct() {
		parent::__destruct();
		} // __destruct()

// static methods
	private static function search_array($keywords,$data,$uri_prefix) {
		$result = array();
		if(isset($data['comment'])) unset($data['comment']);
		foreach($data as $n => $v) {
			$found = true;
			$d = strtolower(($n . ' ' . $v));	// join the name and value
			foreach($keywords as $w) {
				if(stripos($d, $w) === false) {
					$found = false;
					break;
					} // if
				} // foreach
			if($found) {	// make result text
				$result[] = array(
					'url' => $uri_prefix . '&name=' . urlencode($n) . '&keywords=' . urlencode(implode(' ',$keywords)),
					'text' => '<strong>' . Clm::make_nice_name($n) . '</strong>' . ' = ' . preg_replace('/ - -/', '', $v),
					);
				} // if
			} // foreach
		return $result;
		} // search_array()

	private static function join_arrays($arrays) {
		$data = array();
		for($i = 0; $i < count($arrays); $i++) {
			if($arrays[$i]) {
				foreach($arrays[$i] as $n => $v) {
					if(!isset($data[$n])) $data[$n] = $v;
					else  $data[$n] .= ' - ' . $v;
					} // foreach
				} // if
			} // for
		ksort($data);
		return $data;
		} // join_arrays()

	protected static function search_install($keywords) {
		$settings = self::read_ini_settings(false,array('ThemeSettings'));
		$comments = self::read_ini_comments(false,array('ThemeSettings'));	// get control file
		$extras = array( 'DEFAULT_INSTALL_BUTTON' => 'Set all install values to default.',);
		$data = self::join_arrays(array($settings['CommonSettings'], $comments['CommonSettings'], $settings['DataBaseAccess'], $comments['DataBaseAccess'],$extras));
		$result = self::search_array($keywords, $data, 'index.php?action=edit_install');
		$result['title'] = 'Found in Install Settings.';
		return $result;
		} // search_install()

	protected static function search_theme($keywords) {
		$settings = self::read_ini_settings(false,array('CommonSettings','DataBaseAccess'));
		$comments = self::read_ini_comments(false,array('CommonSettings','DataBaseAccess'));	// get control file
		$extras = array( 'DEFAULT_THEME_BUTTON' => 'Set all theme values to default theme.',);
		$data = self::join_arrays(array($settings['ThemeSettings'], $comments['ThemeSettings'],$extras));
		$result = self::search_array($keywords, $data, 'index.php?action=edit_theme');
		$result['title'] = 'Found in Theme Settings.';
		return $result;
		} // search_theme()

	private static function search_DB($keywords,$table,$cols,$uri_prefix) {
		$result = array();
		$sql_query = "SELECT " . implode(',',$cols) .
			" FROM " . $table;
		if(($sql_result = self::$cDB->db_query($sql_query)) &&
			(self::$cDB->db_num_rows($sql_result) > 0)) {
			while($row = self::$cDB->db_fetch_array($sql_result)) {
				$id = $row[$cols[0]];	// [0] has id
				unset($row[$cols[0]]);
				$n = $row[$cols[1]];	// [1] is the key or name
				unset($row[$cols[1]]);
				$v = implode(' - ',$row);
				$found = true;
				$d = strtolower(($n . ' ' . $v));	// join the name and value
				foreach($keywords as $w) {
					if(stripos($d, $w) === false) {
						$found = false;
						break;
						} // if
					} // foreach
				if($found) {	// make result text
					$result[] = array(
						'url' => $uri_prefix . $id . '&name=' . urlencode($n) . '&keywords=' . urlencode(implode(' ',$keywords)),
						'text' => '<strong>' . Clm::make_nice_name($n) . '</strong>' . ' = ' . preg_replace('/ - -/', '', $v),
						);
					} // if
				} // while
			} // if
		return $result;
		} // search_DB()

	protected static function search_users($keywords) {
		$result = array();
		$cols = array('lm_user_id', 'lm_user_name');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_users', $cols, 'index.php?action=edit_users&user_edit_id=');
		$result['title'] = 'Found in Users.';
		return $result;
		} // search_users()

	protected static function search_groups($keywords) {
		$result = array();
		$cols = array('lm_group_id', 'lm_group_name', 'lm_group_description');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_groups', $cols, 'index.php?action=edit_groups&group_edit_id=');
		$result['title'] = 'Found in Groups.';
		return $result;
		} // search_groups()

	protected static function search_sections($keywords) {
		$result = array();
		$cols = array('lm_section_id', 'lm_section_name', 'lm_section_description', 'lm_section_title');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_sections', $cols, 'index.php?action=edit_sections&section_edit_id=');
		$result['title'] = 'Found in Sections.';
		return $result;
		} // search_sections()

	protected static function search_links($keywords) {
		$result = array();
		$cols = array('lm_link_id', 'lm_link_name', 'lm_link_description', 'lm_link_title', 'lm_link_url');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_links', $cols, 'index.php?action=edit_links&link_edit_id=');
		$result['title'] = 'Found in Links.';
		return $result;
		} // search_links()

	protected static function search_tools($keywords) {
		$result = array();
		$cols = array('lm_tool_id', 'lm_tool_name', 'lm_tool_description', 'lm_tool_title', 'lm_tool_url');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_tools', $cols, 'index.php?action=tool&tool_id=');
		$result['title'] = 'Found in Tools.';
		return $result;
		} // search_tools()

	protected static function search_config($keywords) {
		$result = array();
		$cols = array('lm_config_id', 'lm_config_key', 'lm_config_name', 'lm_config_value', 'lm_config_description');	// [0] is the id, [1] is the key
		$result = self::search_DB($keywords, 'lm_configs', $cols, 'index.php?action=edit_config&config_edit_id=');
		$result['title'] = 'Found in Config Settings.';
		return $result;
		} // search_config()

	public static function search($keywords_str) {
		if(empty($keywords_str)) {
			Clm::addMsg(self::help(),'info');
			return array();
			} // if
		$keywords = preg_split('/ /', $keywords_str);
		$results = array('keywords' => $keywords_str);
		$results['users'] = self::search_users($keywords);
		$results['groups'] = self::search_groups($keywords);
		$results['sections'] = self::search_sections($keywords);
		$results['links'] = self::search_links($keywords);
		$results['tools'] = self::search_tools($keywords);
		$results['config'] = self::search_config($keywords);
		$results['install'] = self::search_install($keywords);
		$results['theme'] = self::search_theme($keywords);
		return $results;
		} // search()

	public static function help() {
			$text = "Enter search keywords, seperated by spaces, to search Users, Groups, Sections, Links, Tools, Config, Install and Theme.";
		return $text;
		} // help()

	public static function get_form_results() {
		$results = array();
		if(isset($_POST['search'])) {
			$results = Clm_search::search(((isset($_POST['keywords'])) ? $_POST['keywords']:''));
			} // if
		if(isset($_GET['search'])) {
			$results = Clm_search::search(((isset($_GET['search'])) ? $_GET['search']:''));
			} // if
		return $results;
		} // get_form_results()

	public static function get_form_return_link() {
		$text = '';
		if(isset($_POST['search'])) $text .= '&nbsp;&nbsp;<span class="msg_success"><a href="index.php?action=edit_search&search=' . $_POST['search'] . '" title="Search for: "' . urldecode($_POST['search']) . '">Return to search.</a>';
		return $text;
		} // get_form_return_link()

} // Clm_search
