<?php

/**
 * $Id: lm_xml_sitemap.php 367 2016-04-18 17:02:10Z robert0609 $
 *
 * Description of lm_xml_sitemap
 * Class to generate xml sitemap for links_manager
 *
 * @author robert0609
 */

class Clm_xml_sitemap {

	//put your code here
	protected $db_file = '';

	protected $last_mod_date = '';	// last date the database was modified
	protected $chg_freq = 'weekly';
	protected $chg_priority = '0.6';

	protected $xml_header = '';		// xml header
	protected $xml_main_body = '';	// always included urls
	protected $xml_footer = '';		// xml footer

	protected $href_base = '';

	public $xml_output = '';

	function __construct($db_file = '') {
		if(empty($db_file)) $db_file = LM_FS_SQLITE_DIR . INI_DB_SQLITE_DATABASE;
		if(!$this->init($db_file)) return;
		$this->build_xml_sitemap();
		} // __construct()

	function __destruct() {
		} // __destruct()

	protected function init($db_file) {
		$this->xml_output = '';
		if((!defined('LMC_WEB_SITE_ADDRESS')) ||
			(!defined('INI_XML_SITEMAP_FILE')) ||
			(strlen(INI_XML_SITEMAP_FILE) < 2)) {
			return false;	// not required
			} // if

		if((defined('INI_WEBSITE_CLOSED_BOOL')) &&
			(INI_WEBSITE_CLOSED_BOOL)) {	// remove sitemap
			$xml_file = LM_FS_BASE_DIR . INI_XML_SITEMAP_FILE . '.xml';
			if(is_readable($xml_file)) @unlink($xml_file);
			return false;
			} // if

		if(!is_readable($db_file)) return false; // no db
		$this->db_file = $db_file;

		if(strlen(LMC_WEB_SITE_ADDRESS) > 8) $this->href_base = LMC_WEB_SITE_ADDRESS;
		else $this->href_base = LM_WWW_URL;
		if(substr($this->href_base, -1, 1) != '/') $this->href_base .= '/';
		if(!Clm::has_dns_record($this->href_base)) {
			Clm::addMsg($this->href_base . ' has no DNS record.');
			return false;
			} // if

		$stat = stat($this->db_file);
		$this->last_mod_date = $this->getText(date('Y-m-d' , $stat['mtime']));

		$this->xml_header =
			'<?xml version="1.0" encoding="UTF-8"?>' . "\n" .
			'<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"' . "\n" .
			'	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' . "\n" .
			'	xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9' . "\n" .
			'		http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">' . "\n";

		$this->xml_main_body = $this->encode_url($this->href_base . 'index.php');

		$this->xml_footer =
			'</urlset>' . "\n";

		return true;
		} // init()

	protected function build_xml_sitemap() {
		$this->xml_output = $this->xml_header;
		$this->xml_output .= $this->xml_main_body;

		try {
			$this->add_tools_urls();
			$this->add_body_urls();
			} // try
		catch (Execption $e) {
			Clm::addMsg("Tried and failed to generate sitemap.", 'error');
			$this->xml_output = "";
			} // catch

		$this->xml_output .= $this->xml_footer;
		if($msg = $this->save_xml_stitemap()) {
			Clm::addMsg($msg, 'info');
			return true;
			}
		return false;
		} // build_xml_sitemap()

	protected function add_body_urls() {
		// get data base links
		$sql_section_query = "SELECT  lm_section_id, lm_section_name, lm_section_columns" .
			", lm_section_title,lm_section_description,lm_section_group_ids" .
			", lm_section_image_url" .
			" FROM  lm_sections" .
			" WHERE  lm_section_enabled > 0" .
			" ORDER BY  lm_section_order,lm_section_id";
		$section_cnt = 0; $links_cnt = 0;
		if($result_section = Clm::$cDB->db_query_unbuffered($sql_section_query)) {
			while($section = Clm::$cDB->db_fetch_array_unbuffered($result_section)) {
				if(!$this->is_default_group_id($section['lm_section_group_ids'])) continue;
					$section_cnt++;
					$sql_link_query = "SELECT" .
						" lm_link_id,lm_link_name,lm_link_title,lm_link_description" .
						", lm_link_url, lm_link_image_url, lm_link_new_page, lm_link_add_name2url" .
						", lm_link_added, lm_link_updated" .
						" FROM  lm_links" .
						" WHERE  lm_link_enabled > 0 AND  lm_link_section_id = " . (int)$section['lm_section_id'] .
						" ORDER BY  lm_link_order";

					if($result_link = Clm::$cDB->db_query_unbuffered($sql_link_query)) {
						while($link = Clm::$cDB->db_fetch_array_unbuffered($result_link)) {
							$links_cnt++;
							$this->add_link_url($link);
							} // while
						} // if
					} // while
				} // if
		return true;
		} // add_body_urls()

	protected function add_tools_urls() {
		$sql_query = "SELECT  lm_tool_id,lm_tool_name,lm_tool_url,lm_tool_title" .
			", lm_tool_new_page, lm_tool_enabled, lm_tool_group_ids, lm_tool_add_name2url" .
			", lm_tool_added, lm_tool_updated" .
			" FROM  lm_tools" .
			" WHERE  lm_tool_enabled > 0" .
			" ORDER BY  lm_tool_order,lm_tool_name,lm_tool_id";
		if(($result = Clm::$cDB->db_query($sql_query)) &&
			(Clm::$cDB->db_num_rows($result) > 0)) {
			$row = 0;
			// Clm::page_start_comment(__FILE__);
			while($tool = Clm::$cDB->db_fetch_array($result)) {
				if(!$this->is_default_group_id($tool['lm_tool_group_ids'])) continue;
				$this->add_local_url('index.php?action=tool&tool_id=' . (int)$tool['lm_tool_id'] . '&name=' . urlencode($tool['lm_tool_name']),$tool['lm_tool_updated']);
				} // while
			Clm::$cDB->db_free_result($result);
			} // if
		return true;
		} // add_tools_urls()

	protected function add_link_url($link) {
		if(empty($link)) return false;
		if($link['lm_link_new_page']) {
			$this->xml_output .= $this->encode_url($link['lm_link_url'] . ((int)$link['lm_link_add_name2url'] ? '?name=' . urlencode($link['lm_link_name']):''));
			} // if
		else {
			$this->add_local_url('index.php?action=link&link_id=' . (int)$link['lm_link_id'] . '&name=' . urlencode($link['lm_link_name']),$link['lm_link_updated']);
			} // else
		return true;
		} // add_link_url()

	protected function is_default_group_id($group_ids, $default_id = 0) {
		// if(!Clm::check_user_group_ids($group_ids)) return false;	// test

		if(strlen($group_ids) == 0) return false;
		$op_group_ids = explode(':',$group_ids);
		if(in_array($default_id,$op_group_ids)) return true;
		return false;
		} // is_default_group_id()

	protected function encode_url($url,$datetime = '') {
		if(empty($datetime)) $date = $this->last_mod_date;
		else $date = preg_replace ('/ \d\d:\d\d:\d\d/', '', $datetime);
		if(!preg_match('/http:|https:|ftp:/', $url)) $url = ($this->href_base . $url);
		$xml =	'	<url>' . "\n" .
				'		<loc>' . $this->getText($url) . '</loc>' . "\n" .
				'		<lastmod>' . $date . '</lastmod>' . "\n" .
				'		<changefreq>' . $this->chg_freq . '</changefreq>' . "\n" .
				'		<priority>' . $this->chg_priority . '</priority>' . "\n" .
				'	</url>' . "\n";
		return $xml;
		} // encode_url()

	protected function add_local_url($url,$datetime) {
		$this->xml_output .= $this->encode_url($url,$datetime);
		} // add_local_url()

	protected function getText($text) {	// clean xml content to meet xml text standard
		$sub = '';
		$remove = array("\r\n","\n\r","\r","\n");
		$text = str_replace($remove,$sub, $text);

		$text = htmlentities($text);	// @TODO check??? not sure about this
		// http://www.sitemaps.org/protocol.html#sitemapXMLExample says this should be done

		return $text;
		} // getText()

	protected function save_xml_stitemap() {
		$msg ='';
		$xml_file = LM_FS_BASE_DIR . INI_XML_SITEMAP_FILE . '.xml';
		if(empty($this->xml_output)) {
			if(is_readable($xml_file)) unlink($xml_file);
			$msg = 'Deleted sitemap "' . $xml_file . '" due to errors.';
			return $msg;
			} // if
		$xml_http = ' (Link=<a href="' . INI_XML_SITEMAP_FILE . '.xml"' . ' target="_blank"' . '>' . INI_XML_SITEMAP_FILE . '.xml</a>)';
		if(is_readable($xml_file)) $msg = 'Updated sitemap "' . $xml_file . '"' . $xml_http;
		else $msg = 'Created sitemap "' . $xml_file . '"' . $xml_http;
		$h = fopen($xml_file,'w');
		if(!$h) {
			Clm::$cDB->logEvent('WARNING: Failed to saved sitemap in "' . basename($xml_file) . '".');
			return 'Failed to save sitemap "' . $xml_file . '"';
			} // if
		fwrite($h,  $this->xml_output);
		fclose($h);
		Clm::$cDB->logEvent('INFO: Saved sitemap in "' . basename($xml_file) . '".');
		@chmod($xml_file, 0664);
		@chgrp($xml_file,get_current_user());
		return $msg;
		} // save_xml_stitemap()

} // Clm_xml_sitemap
