<?php
/**
 * $Id: msgs.php 349 2015-07-04 02:21:39Z robert0609 $
 *
 * Description of Cmsgs
 *
 * A basic storage class.
 * Accumulates all messages avoiding html header contaminaion,
 * then messages can be formatted and output to the page in an orderly fashion.
 *
 * Remaining/undisplayed messages are saved to $_SESSION
 *
 * @author robert0609
 */

class Cmsgs {

	//put your code here
	private static $m_saMsgs = false;

	function __construct() {
		} // __construct()

	function __destruct() {
		self::saveMsgs();
		} // __destruct()

	protected static function initMsgs() {
		if(!is_array(self::$m_saMsgs)) {
			self::$m_saMsgs = array();	// start
			} // if
		if((isset($_SESSION['msgs'])) && (is_array($_SESSION['msgs']))) {
			self::$m_saMsgs = array_merge(self::$m_saMsgs,$_SESSION['msgs']);
			unset($_SESSION['msgs']);
			} // if
		} // initMsgs()

	 public static function getMsgs($clearFlg = true) {
		self::initMsgs();
		if(!self::$m_saMsgs) return false;
		$text = "\n";
		for($i = 0, $n = count(self::$m_saMsgs); $i < $n; $i++) {
			$msg = self::$m_saMsgs[$i];
			switch($msg['type']) {
			case 'warning':
				$pre = 'WARNING:';
				$class = 'msg_warning';
				$img = 'warning.gif';
				break;
			case 'success':
				$pre = 'SUCCESS:';
				$class = 'msg_success';
				$img = 'success.gif';
				break;
			case 'info':
				$pre = 'INFO:';
				$class = 'msg_info';
				$img = 'info.gif';
				break;
			default:
			case 'error':
				$pre = 'ERROR:';
				$class = 'msg_error';
				$img = 'error.gif';
				break;
				} // switch

			if($i > 0) $text .= '<br>';
			$text .= '<span class="' . $class . '">';
			$text .= '<img src="images/' . $img . '">';
			$text .= '&nbsp;' . $pre . ' ' . $msg['msg'];
			$text .= '</span>' . "\n";
			} // for

		if($clearFlg) self::$m_saMsgs = false;
		return $text;
		} // getMsgs()

	public static function getMsgsCount($type = 'all') {
		self::initMsgs();
		$type = strtolower($type);
		if(!self::$m_saMsgs) return 0;
		if($type == 'all') return count(self::$m_saMsgs);
		$cnt = 0;
		for($i = 0, $n = count(self::$m_saMsgs); $i < $n; $i++) {
			if(self::$m_saMsgs[$i]['type'] == $type) $cnt++;
			} // for
		return $cnt;
		} // getMsgsCount()

	public static function addMsg($msg, $type = 'error') {	// allows msgs to be stored to be show in a meaning full way
		self::initMsgs();
		self::$m_saMsgs[] = array(
			'msg' => $msg,
			'type' => strtolower($type),
			);
		} // addMsg()

	public static function saveMsgs() {
		if((is_array(self::$m_saMsgs)) && (!empty(self::$m_saMsgs)) && (isset($_SESSION))) {
			$_SESSION['msgs'] = self::$m_saMsgs;
			}
		} // saveMsgs()

} // Cmsgs
