// $Id: title_tooltips.js.php 272 2015-03-19 05:22:07Z robert0609 $
// Copyright BRAEWORKS (R) 2014
// tooltip js code

var tt_idx = 0;
var titletipOps = [];	// place to save Ctitle2tooltip iterations
var tablet_flg = false;	// controls browser display
var Ctitle2tooltip = function(el,type,tablet_flg) {
	var elem = el;
	var titletip = false;
	var h_margin = 5;
	var v_margin = 5;
	var tablet = tablet_flg;
	var title_text = elem.getAttribute("title");

	this.titletipShow = function (evt) {

		// get window client are
		var w = window.innerWidth;
		var h = window.innerHeight;

		// get the position of the hover element
		var evtBox = evt.target.getBoundingClientRect();
		var ttBox = titletip.getBoundingClientRect();

		// adjust titletip position
		var h_offset;
		if(evtBox.left < (w / 2)) {	// left half of window
			h_offset = evtBox.left + (evtBox.width / 2) + h_margin;
			} // if
		else {	// right half of window
			h_offset = evtBox.left + (evtBox.width / 2) - ttBox.width - h_margin;
			} // else
		titletip.style.left = h_offset.toString() + "px";

		var v_offset;
		if(evtBox.top < (h / 2)) {	// top half of window
			v_offset = evtBox.top + evtBox.height + v_margin;
			} // if
		else {	// bottom half of window
			v_offset = evtBox.top - ttBox.height - v_margin;
			} // else
		titletip.style.top = v_offset.toString() + "px";

		// make titletip VISIBLE
		titletip.style.visibility="visible";
		titletip.style.zIndex="10000";
	} // titletipShow()

	this.titletipHide = function(evt) {
		titletip.style.visibility="hidden";
		} // titletipHide()

	this.titletipCheck = function() {
		if(!titletip) return false;
		if(!titletip.parentElement.nodeName.length) {
			delete titletip;
			return false;
			} // if
		return true; // ok
		} // titletipCheck()

	if (title_text != null) {
		// create balloon element to display info
		titletip = document.createElement("div");

		// add titletip content now to get sizes
		titletip.innerHTML = '<span class="title_tooltip" ' + (tablet ? ' onclick="this.style.visibility=\'hidden\'";':'') + '>' + title_text.replace(/\\n|&#10;/gi,'<br>') + '</span>';

		// set class
		titletip.classname="title_tooltip";

	// set style
		titletip.style.visibility="hidden";
		titletip.style.position="fixed";
		titletip.style.top="1ex";
		titletip.style.left="1ex";

		// insert into DOM
		var txt_node = document.createTextNode('');
		titletip.appendChild(txt_node);
		document.body.insertBefore(titletip, document.body.firstChild);

		if(!tablet) {
			// assign handler
			elem.addEventListener("mouseover", this.titletipShow , false);
			elem.addEventListener("mouseout", this.titletipHide , false);
			} // if
		else { // a tablet, ipad, etc
			elem.addEventListener("onclick", this.titletipShow , false);
			} // else

		// alert(title_text);
		elem.setAttribute('title','');	// turn off old title
		} // if
	};	// Ctitle2tooltip


function titles2tooltips(tablet) {
	var elems2chk = ["h1","h2","h3","a","p","input","button","select","textarea","img","td","th","tr","div","span"];
	if(tablet) tablet_flg = tablet;
	for(var el = 0; el < elems2chk.length; el++) {
		var elems = document.getElementsByTagName(elems2chk[el]);
		for (var i=0; i< elems.length; i++) {
			var title_text = elems[i].getAttribute("title");
			if ((title_text != null) && (title_text.length > 0)) {
				titletipOps[tt_idx++] = new Ctitle2tooltip(elems[i],elems2chk[el],tablet_flg);	// stick it
				} // if
			} // for
		} // for
	} // titles2tooltips()

function reRunTitles2tooltips(tablet) {
	// remove dead elements from ajax replacements, should not need to do this
	var n = titletipOps.length;
	for(var i = 0; i < n; i++) {
		if(!titletipOps[i]) continue;
		if(titletipOps[i] == i) continue;
		if(!titletipOps[i].titletipCheck()) {
			delete titletipOps[i];	// delete it
			titletipOps[i] = i;	// replace with marker
			} // if
		} // for
	titles2tooltips(tablet);
	} // reRunTitles2tooltips()

